# -*- coding: UTF-8 -*-
"""Dataset Title Helper Tests"""
from datetime import datetime

from hdx.data.dataset_title_helper import DatasetTitleHelper


class TestDatasetTitleHelper:
    def test_fuzzy_match_dates_in_title(self):
        ignore_wrong_years = list()
        ranges = list()
        assert DatasetTitleHelper.fuzzy_match_dates_in_title('Myanmar Town July 2019', ranges,
                                                             ignore_wrong_years) == 'Myanmar Town'
        assert ranges == [(datetime(2019, 7, 1, 0, 0), datetime(2019, 7, 31, 0, 0))]
        ranges = list()
        assert DatasetTitleHelper.fuzzy_match_dates_in_title('Myanmar Town 2019 July', ranges,
                                                             ignore_wrong_years) == 'Myanmar Town'
        assert ranges == [(datetime(2019, 7, 1, 0, 0), datetime(2019, 7, 31, 0, 0))]

    def test_get_date_from_title(self):
        assert DatasetTitleHelper.get_dates_from_title('Myanmar Self Administered Regions Boundaries MIMU v9.2.1') == (
            'Myanmar Self Administered Regions Boundaries MIMU v9.2.1', list())
        assert DatasetTitleHelper.get_dates_from_title(
            'Myanmar Town 2019 July') == ('Myanmar Town', [(datetime(2019, 7, 1, 0, 0), datetime(2019, 7, 31, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title('Formal Sector School Location Upper Myanmar (  2019   )') == (
            'Formal Sector School Location Upper Myanmar', [(datetime(2019, 1, 1, 0, 0), datetime(2019, 12, 31, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title('ICA Armenia, 2017 - Drought Risk, 1981-2015') == (
            'ICA Armenia - Drought Risk', [(datetime(1981, 1, 1, 0, 0), datetime(2015, 12, 31, 0, 0)),
                                           (datetime(2017, 1, 1, 0, 0), datetime(2017, 12, 31, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title('ICA Sudan, 2018 - Land Degradation, 2001-2013') == (
            'ICA Sudan - Land Degradation', [(datetime(2001, 1, 1, 0, 0), datetime(2013, 12, 31, 0, 0)),
                                             (datetime(2018, 1, 1, 0, 0), datetime(2018, 12, 31, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title('Central African Republic, Bridges, January 2019') == (
            'Central African Republic, Bridges', [(datetime(2019, 1, 1, 0, 0), datetime(2019, 1, 31, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title(
            'Afghanistan:District Accessibility for WFP and Partners Staff as of 05 May 2019') == (
                   'Afghanistan:District Accessibility for WFP and Partners Staff',
                   [(datetime(2019, 5, 5, 0, 0), datetime(2019, 5, 5, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title('Tanintharyi Region Land Cover - March 2016 (Original)') == (
            'Tanintharyi Region Land Cover (Original)', [(datetime(2016, 3, 1, 0, 0), datetime(2016, 3, 31, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title(
            'Kachin State and Sagaing Region 2002-2014 Forest Cover Change') == (
                   'Kachin State and Sagaing Region Forest Cover Change',
                   [(datetime(2002, 1, 1, 0, 0), datetime(2014, 12, 31, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title('Ward boundaries Yangon City_mimu_v8_1') == (
            'Ward boundaries Yangon City_mimu_v8_1', list())
        assert DatasetTitleHelper.get_dates_from_title('Mon_State_Village_Tract_Boundaries') == (
            'Mon_State_Village_Tract_Boundaries', list())
        assert DatasetTitleHelper.get_dates_from_title('ICA Afghanistan, 2019 - Landslide hazard, 2013') == (
            'ICA Afghanistan - Landslide hazard', [(datetime(2013, 1, 1, 0, 0), datetime(2013, 12, 31, 0, 0)),
                                                   (datetime(2019, 1, 1, 0, 0), datetime(2019, 12, 31, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title(
            'Afghanistan Percentage of Food Insecure Population Based on Combined Food Consumption Score and Coping Strategy Index by Province - ALCS 2013/14') == (
                   'Afghanistan Percentage of Food Insecure Population Based on Combined Food Consumption Score and Coping Strategy Index by Province - ALCS',
                   [(datetime(2013, 1, 1, 0, 0), datetime(2014, 12, 31, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title('ALCS 2000/10') == (
            'ALCS', [(datetime(2000, 10, 1, 0, 0), datetime(2000, 10, 31, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title('ALCS 2014/13') == (  # not a month and range going down
            'ALCS 2014/13', list())
        assert DatasetTitleHelper.get_dates_from_title(
            'Mon_State_Village_Tract_Boundaries 9999') == ('Mon_State_Village_Tract_Boundaries 9999', list())
        assert DatasetTitleHelper.get_dates_from_title('Mon_State_Village_Tract_Boundaries 10/12/01 lala') == (
            'Mon_State_Village_Tract_Boundaries 10/12/01 lala', list())
        # It's the "Mon" that makes an extra date component that causes it to ignore the date (correctly)
        assert DatasetTitleHelper.get_dates_from_title('State_Village_Tract_Boundaries 10/12/01 lala') == (
            'State_Village_Tract_Boundaries lala', [(datetime(2001, 12, 10, 0, 0), datetime(2001, 12, 10, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title(
            'Crops production (2016) - Tajikistan Vulnerability & Resilience Atlas, 2019') == (
                   'Crops production - Tajikistan Vulnerability & Resilience Atlas',
                   [(datetime(2016, 1, 1, 0, 0), datetime(2016, 12, 31, 0, 0)),
                    (datetime(2019, 1, 1, 0, 0), datetime(2019, 12, 31, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title('Location of partners as of Feb. 5, 2019') == (
            'Location of partners', [(datetime(2019, 2, 5, 0, 0), datetime(2019, 2, 5, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title('ICA Armenia, 2016 & 2017 - Land Degradation, 2001-2012') == (
            'ICA Armenia - Land Degradation', [(datetime(2001, 1, 1, 0, 0), datetime(2012, 12, 31, 0, 0)),
                                               (datetime(2016, 1, 1, 0, 0), datetime(2017, 12, 31, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title(
            'ICA Afghanistan, 2016 - Food Insecurity Risk, 2007/08-2014') == (
                   'ICA Afghanistan - Food Insecurity Risk',
                   [(datetime(2007, 8, 1, 0, 0), datetime(2014, 12, 31, 0, 0)),
                    (datetime(2016, 1, 1, 0, 0), datetime(2016, 12, 31, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title('Risk, 2020/19-2014') == (
            'Risk, 2020/19', [(datetime(2014, 1, 1, 0, 0), datetime(2014, 12, 31, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title('south sudan access constraints shp for 20200124') == (
            'south sudan access constraints shp', [(datetime(2020, 1, 24, 0, 0), datetime(2020, 1, 24, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title('Cambodia Flood Extent in 2011') == (
            'Cambodia Flood Extent', [(datetime(2011, 1, 1, 0, 0), datetime(2011, 12, 31, 0, 0))])
        assert DatasetTitleHelper.get_dates_from_title(
            'Access: Proportion of the population consuming less than 2100 kcal per day (average of 2011-2013), National Statistics Committee 2013') == (
                   'Access: Proportion of the population consuming less than 2100 kcal per day (average), National Statistics Committee',
                   [(datetime(2011, 1, 1, 0, 0), datetime(2013, 12, 31, 0, 0)),
                    (datetime(2013, 1, 1, 0, 0), datetime(2013, 12, 31, 0, 0))])



# -*- coding: UTF-8 -*-
"""Resource view Tests"""
import copy
import json
from os.path import join

import pytest
from hdx.utilities.dictandlist import merge_two_dictionaries

from hdx.data.hdxobject import HDXError
from hdx.data.resource_view import ResourceView
from hdx.hdx_configuration import Configuration
from . import MockResponse

hxl_preview_config = '{"configVersion":2,"bites":[{"init":true,"type":"key figure","filteredValues":[],"errorMsg":null,"ingredient":{"aggregateColumn":null,"valueColumn":"#affected+killed","aggregateFunction":"sum"},"dataTitle":"#affected+killed","displayCategory":"Key Figures","unit":null,"hashCode":-1955043658,"title":"Sum of fatalities","value":null},{"init":true,"type":"chart","filteredValues":[],"errorMsg":null,"swapAxis":true,"showGrid":true,"pieChart":false,"ingredient":{"aggregateColumn":"#adm1+name","valueColumn":"#affected+killed","aggregateFunction":"sum"},"dataTitle":"#affected+killed","displayCategory":"Charts","hashCode":738289179,"title":"Sum of fatalities grouped by admin1","values":null,"categories":null},{"init":true,"type":"chart","filteredValues":[],"errorMsg":null,"swapAxis":true,"showGrid":true,"pieChart":false,"ingredient":{"aggregateColumn":"#adm2+name","valueColumn":"#affected+killed","aggregateFunction":"sum"},"dataTitle":"#affected+killed","displayCategory":"Charts","hashCode":766918330,"title":"Sum of fatalities grouped by admin2","values":null,"categories":null}]}'

resource_view_list = [{'description': '', 'resource_id': '25982d1c-f45a-45e1-b14e-87d367413045', 'view_type': 'recline_view',
                       'title': 'Data Explorer', 'package_id': '53f4375e-8872-4bcd-9746-c0fda941dadb', 'id': 'd80301b5-4abd-49bd-bf94-fa4af7b6e7a4'},
                      {'description': '', 'resource_id': '25982d1c-f45a-45e1-b14e-87d367413045',
                       'hxl_preview_config': hxl_preview_config,
                       'view_type': 'hdx_hxl_preview', 'title': 'Quick Charts', 'package_id': '53f4375e-8872-4bcd-9746-c0fda941dadb', 'id': 'c06b5a0d-1d41-4a74-a196-41c251c76023'}]

resultdict = resource_view_list[1]


def resource_view_mockshow(url, datadict):
    if 'show' not in url:
        return MockResponse(404,
                            '{"success": false, "error": {"message": "TEST ERROR: Not show", "__type": "TEST ERROR: Not Show Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_show"}')
    result = json.dumps(resultdict)
    if datadict['id'] == 'c06b5a0d-1d41-4a74-a196-41c251c76023' or datadict['id'] == 'MyResourceView1':
        return MockResponse(200,
                            '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_show"}' % result)
    if datadict['id'] == 'TEST2':
        return MockResponse(404,
                            '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_show"}')
    if datadict['id'] == 'TEST3':
        return MockResponse(200,
                            '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_show"}')
    return MockResponse(404,
                        '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_show"}')


def resource_view_mocklist(url, datadict):
    if 'list' not in url:
        return MockResponse(404,
                            '{"success": false, "error": {"message": "TEST ERROR: Not all", "__type": "TEST ERROR: Not All Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_list"}')
    if datadict['id'] == '25982d1c-f45a-45e1-b14e-87d367413045':
        return MockResponse(200,
                            '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_list"}' % json.dumps(resource_view_list))
    return MockResponse(404,
                        '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_list"}')


def resource_view_mockcreate(url, datadict):
    if 'create' not in url:
        return MockResponse(404,
                            '{"success": false, "error": {"message": "TEST ERROR: Not create", "__type": "TEST ERROR: Not Create Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_create"}')
    if datadict['title'] == 'A Preview':
        result = json.dumps(resultdict)
        return MockResponse(200,
                            '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_create"}' % result)
    if datadict['title'] == 'Quick Charts':
        resultdictcopy = copy.deepcopy(resultdict)
        result = json.dumps(merge_two_dictionaries(resultdictcopy, datadict))
        return MockResponse(200,
                            '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_create"}' % result)
    if datadict['title'] == 'XXX':
        return MockResponse(404,
                            '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_create"}')
    if datadict['title'] == 'YYY':
        return MockResponse(200,
                            '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_create"}')

    return MockResponse(404,
                        '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_create"}')


class TestResourceView:
    resource_view_data = {
        'resource_id': '25982d1c-f45a-45e1-b14e-87d367413045',
        'title': 'Data Explorer',
        'view_type': 'recline_view',
    }

    @pytest.fixture(scope='class')
    def static_yaml(self):
        return join('tests', 'fixtures', 'config', 'hdx_resource_view_static.yml')

    @pytest.fixture(scope='class')
    def static_json(self):
        return join('tests', 'fixtures', 'config', 'hdx_resource_view_static.json')

    @pytest.fixture(scope='function')
    def read(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                datadict = json.loads(data.decode('utf-8'))
                return resource_view_mockshow(url, datadict)

        Configuration.read().remoteckan().session = MockSession()

    @pytest.fixture(scope='function')
    def post_create(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                datadict = json.loads(data.decode('utf-8'))
                if 'show' in url:
                    return resource_view_mockshow(url, datadict)
                if 'list' in url:
                    return resource_view_mocklist(url, datadict)
                if 'create' in url:
                    return resource_view_mockcreate(url, datadict)
                return MockResponse(404,
                                    '{"success": false, "error": {"message": "TEST ERROR: Not create", "__type": "TEST ERROR: Not Create Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_create"}')

        Configuration.read().remoteckan().session = MockSession()

    @pytest.fixture(scope='function')
    def post_update(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                datadict = json.loads(data.decode('utf-8'))
                if 'show' in url:
                    return resource_view_mockshow(url, datadict)
                if 'list' in url:
                    return resource_view_mocklist(url, datadict)
                if 'update' not in url:
                    return MockResponse(404,
                                        '{"success": false, "error": {"message": "TEST ERROR: Not update", "__type": "TEST ERROR: Not Update Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_update"}')
                resultdictcopy = copy.deepcopy(resultdict)
                merge_two_dictionaries(resultdictcopy, datadict)

                result = json.dumps(resultdictcopy)
                if datadict['title'] == 'Quick Charts':
                    return MockResponse(200,
                                        '{"success": true, "result": %s, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_update"}' % result)
                if datadict['title'] == 'XXX':
                    return MockResponse(404,
                                        '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_update"}')
                if datadict['title'] == 'YYY':
                    return MockResponse(200,
                                        '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_update"}')

                return MockResponse(404,
                                    '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_update"}')

        Configuration.read().remoteckan().session = MockSession()

    @pytest.fixture(scope='function')
    def post_delete(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                decodedata = data.decode('utf-8')
                datadict = json.loads(decodedata)
                if 'show' in url:
                    return resource_view_mockshow(url, datadict)
                if 'delete' not in url:
                    return MockResponse(404,
                                        '{"success": false, "error": {"message": "TEST ERROR: Not delete", "__type": "TEST ERROR: Not Delete Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_delete"}')
                if datadict['id'] == 'c06b5a0d-1d41-4a74-a196-41c251c76023':
                    return MockResponse(200,
                                        '{"success": true, "result": null, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_delete"}')

                return MockResponse(404,
                                    '{"success": false, "error": {"message": "Not found", "__type": "Not Found Error"}, "help": "http://test-data.humdata.org/api/3/action/help_show?name=resource_view_delete"}')

        Configuration.read().remoteckan().session = MockSession()

    @pytest.fixture(scope='function')
    def post_list(self):
        class MockSession(object):
            @staticmethod
            def post(url, data, headers, files, allow_redirects, auth=None):
                datadict = json.loads(data.decode('utf-8'))
                return resource_view_mocklist(url, datadict)

        Configuration.read().remoteckan().session = MockSession()

    def test_read_from_hdx(self, configuration, read):
        resource_view = ResourceView.read_from_hdx('c06b5a0d-1d41-4a74-a196-41c251c76023')
        assert resource_view['id'] == 'c06b5a0d-1d41-4a74-a196-41c251c76023'
        assert resource_view['title'] == 'Quick Charts'
        resource_view = ResourceView.read_from_hdx('TEST2')
        assert resource_view is None
        resource_view = ResourceView.read_from_hdx('TEST3')
        assert resource_view is None

    def test_create_in_hdx(self, configuration, post_create):
        resource_view = ResourceView()
        with pytest.raises(HDXError):
            resource_view.create_in_hdx()
        resource_view['title'] = 'Data Explorer'
        with pytest.raises(HDXError):
            resource_view.create_in_hdx()

        data = copy.deepcopy(self.resource_view_data)
        resource_view = ResourceView(data)
        resource_view['title'] = 'A Preview'
        resource_view.create_in_hdx()
        assert resource_view['id'] == 'c06b5a0d-1d41-4a74-a196-41c251c76023'
        assert resource_view['view_type'] == 'hdx_hxl_preview'
        assert 'state' not in resource_view

        data['title'] = 'XXX'
        resource_view = ResourceView(data)
        with pytest.raises(HDXError):
            resource_view.create_in_hdx()

        data['title'] = 'YYY'
        resource_view = ResourceView(data)
        with pytest.raises(HDXError):
            resource_view.create_in_hdx()

    def test_update_in_hdx(self, configuration, post_update):
        resource_view = ResourceView()
        resource_view['id'] = 'NOTEXIST'
        with pytest.raises(HDXError):
            resource_view.update_in_hdx()
        resource_view['id'] = 'LALA'
        with pytest.raises(HDXError):
            resource_view.update_in_hdx()

        resource_view = ResourceView.read_from_hdx('c06b5a0d-1d41-4a74-a196-41c251c76023')
        assert resource_view['id'] == 'c06b5a0d-1d41-4a74-a196-41c251c76023'
        assert resource_view['view_type'] == 'hdx_hxl_preview'

        resource_view['id'] = 'c06b5a0d-1d41-4a74-a196-41c251c76023'
        resource_view['view_type'] = 'recline_view'
        resource_view['resource_id'] = 'LALA'
        resource_view.update_in_hdx()
        assert resource_view['id'] == 'c06b5a0d-1d41-4a74-a196-41c251c76023'
        assert resource_view['view_type'] == 'recline_view'
        assert resource_view['resource_id'] == 'LALA'
        assert 'state' not in resource_view

        resource_view['id'] = 'NOTEXIST'
        with pytest.raises(HDXError):
            resource_view.update_in_hdx()

        resource_view['view_type'] = 'hdx_hxl_preview'
        resource_view['resource_id'] = '25982d1c-f45a-45e1-b14e-87d367413045'
        resource_view.update_in_hdx()
        assert resource_view['id'] == 'NOTEXIST'
        assert resource_view['view_type'] == 'hdx_hxl_preview'
        assert resource_view['resource_id'] == '25982d1c-f45a-45e1-b14e-87d367413045'

        del resource_view['id']
        resource_view['title'] = 'NOTEXIST'
        with pytest.raises(HDXError):
            resource_view.update_in_hdx()

        data = copy.deepcopy(self.resource_view_data)
        data['id'] = 'c06b5a0d-1d41-4a74-a196-41c251c76023'
        data['title'] = 'Quick Charts'
        data['description'] = 'Custom chart X'
        resource_view = ResourceView(data)
        resource_view.create_in_hdx()
        assert resource_view['id'] == 'c06b5a0d-1d41-4a74-a196-41c251c76023'
        assert resource_view['description'] == 'Custom chart X'
        assert resource_view['view_type'] == 'recline_view'
        assert 'state' not in resource_view

    def test_delete_from_hdx(self, configuration, post_delete):
        resource_view = ResourceView.read_from_hdx('c06b5a0d-1d41-4a74-a196-41c251c76023')
        resource_view.delete_from_hdx()
        resource_view = ResourceView.read_from_hdx('c06b5a0d-1d41-4a74-a196-41c251c76023')
        del resource_view['id']
        with pytest.raises(HDXError):
            resource_view.delete_from_hdx()

    def test_update_yaml(self, configuration, static_yaml):
        data = copy.deepcopy(self.resource_view_data)
        resource_view = ResourceView(data)
        assert resource_view['view_type'] == 'recline_view'
        assert resource_view['title'] == 'Data Explorer'
        resource_view.update_from_yaml(static_yaml)
        assert resource_view['view_type'] == 'hdx_hxl_preview'
        assert resource_view['title'] == 'Quick Charts'
        assert resource_view['description'] == 'lala'
        assert resource_view['resource_id'] == '25982d1c-f45a-45e1-b14e-87d367413045'

    def test_update_json(self, configuration, static_json):
        data = copy.deepcopy(self.resource_view_data)
        resource_view = ResourceView(data)
        assert resource_view['view_type'] == 'recline_view'
        assert resource_view['title'] == 'Data Explorer'
        resource_view['view_type'] = 'hdx_hxl_preview'
        resource_view.update_from_json(static_json)
        assert resource_view['view_type'] == 'recline_view'
        assert resource_view['title'] == 'Data Explorer'
        assert resource_view['description'] == 'haha'
        assert resource_view['resource_id'] == '25982d1c-f45a-45e1-b14e-87d367413045'

    def test_copy(self, configuration, read):
        data = copy.deepcopy(self.resource_view_data)
        resource_view = ResourceView(data)
        resource_view.copy(resultdict)
        assert resource_view['resource_id'] == self.resource_view_data['resource_id']
        assert resource_view['view_type'] == 'hdx_hxl_preview'
        assert resource_view['hxl_preview_config'] == hxl_preview_config
        data = copy.deepcopy(self.resource_view_data)
        resource_view = ResourceView(data)
        resource_view.copy('c06b5a0d-1d41-4a74-a196-41c251c76023')
        assert resource_view['resource_id'] == self.resource_view_data['resource_id']
        assert resource_view['view_type'] == 'hdx_hxl_preview'
        assert resource_view['hxl_preview_config'] == hxl_preview_config
        with pytest.raises(HDXError):
            resource_view.copy('123')
        with pytest.raises(HDXError):
            resource_view.copy(5)

    def test_get_all_for_resource(self, configuration, post_list):
        resource_views = ResourceView.get_all_for_resource('25982d1c-f45a-45e1-b14e-87d367413045')
        assert resource_views[0]['id'] == 'd80301b5-4abd-49bd-bf94-fa4af7b6e7a4'
        assert resource_views[1]['id'] == 'c06b5a0d-1d41-4a74-a196-41c251c76023'



# -*- coding: utf-8 -*-
"""Configuration for HDX"""
import os

import requests
import six
from hdx.utilities.email import Email
from hdx.utilities.session import get_session
from hdx.utilities.useragent import UserAgent, UserAgentError

from hdx.version import get_api_version

if six.PY2:
    from UserDict import IterableUserDict as UserDict
else:
    from collections import UserDict


import logging
from base64 import b64decode
from os.path import expanduser, join, isfile
from typing import Optional, Any, Dict, Tuple

import ckanapi

from hdx.utilities.dictandlist import merge_two_dictionaries
from hdx.utilities.loader import load_yaml, load_json
from hdx.utilities.path import script_dir_plus_file

logger = logging.getLogger(__name__)


class ConfigurationError(Exception):
    pass


class Configuration(UserDict, object):
    """Configuration for HDX

    Args:
        **kwargs: See below
        user_agent (str): User agent string. HDXPythonLibrary/X.X.X- is prefixed. Must be supplied if remoteckan is not.
        user_agent_config_yaml (str): Path to YAML user agent configuration. Ignored if user_agent supplied. Defaults to ~/.useragent.yml.
        user_agent_lookup (str): Lookup key for YAML. Ignored if user_agent supplied.
        hdx_url (str): HDX url to use. Overrides hdx_site.
        hdx_site (str): HDX site to use eg. prod, test.
        hdx_read_only (bool): Whether to access HDX in read only mode. Defaults to False.
        hdx_key (str): Your HDX key. Ignored if hdx_read_only = True.
        hdx_config_dict (dict): HDX configuration dictionary to use instead of above 3 parameters OR
        hdx_config_json (str): Path to JSON HDX configuration OR
        hdx_config_yaml (str): Path to YAML HDX configuration
        project_config_dict (dict): Project configuration dictionary OR
        project_config_json (str): Path to JSON Project configuration OR
        project_config_yaml (str): Path to YAML Project configuration
        hdx_base_config_dict (dict): HDX base configuration dictionary OR
        hdx_base_config_json (str): Path to JSON HDX base configuration OR
        hdx_base_config_yaml (str): Path to YAML HDX base configuration. Defaults to library's internal hdx_base_configuration.yml.
    """

    _configuration = None
    home_folder = expanduser('~')
    default_hdx_base_config_yaml = script_dir_plus_file('hdx_base_configuration.yml', ConfigurationError)
    default_hdx_config_yaml = join(home_folder, '.hdx_configuration.yml')

    apiversion = get_api_version()
    prefix = 'HDXPythonLibrary/%s' % apiversion

    def __init__(self, **kwargs):
        # type: (Any) -> None
        super(Configuration, self).__init__()

        self._remoteckan = None
        self._emailer = None

        hdx_base_config_found = False
        hdx_base_config_dict = kwargs.get('hdx_base_config_dict', None)
        if hdx_base_config_dict:
            hdx_base_config_found = True
            logger.info('Loading HDX base configuration from dictionary')

        hdx_base_config_json = kwargs.get('hdx_base_config_json', '')
        if hdx_base_config_json:
            if hdx_base_config_found:
                raise ConfigurationError('More than one HDX base configuration given!')
            hdx_base_config_found = True
            logger.info('Loading HDX base configuration from: %s' % hdx_base_config_json)
            hdx_base_config_dict = load_json(hdx_base_config_json)

        hdx_base_config_yaml = kwargs.get('hdx_base_config_yaml', '')
        if hdx_base_config_found:
            if hdx_base_config_yaml:
                raise ConfigurationError('More than one HDX base configuration given!')
        else:
            if not hdx_base_config_yaml:
                hdx_base_config_yaml = Configuration.default_hdx_base_config_yaml
                logger.info('No HDX base configuration parameter. Using default base configuration file: %s.' % hdx_base_config_yaml)
            logger.info('Loading HDX base configuration from: %s' % hdx_base_config_yaml)
            hdx_base_config_dict = load_yaml(hdx_base_config_yaml)

        hdx_config_found = False
        hdx_config_dict = kwargs.get('hdx_config_dict', None)
        if hdx_config_dict:
            hdx_config_found = True
            logger.info('Loading HDX configuration from dictionary')

        hdx_config_json = kwargs.get('hdx_config_json', '')
        if hdx_config_json:
            if hdx_config_found:
                raise ConfigurationError('More than one HDX configuration given!')
            hdx_config_found = True
            logger.info('Loading HDX configuration from: %s' % hdx_config_json)
            hdx_config_dict = load_json(hdx_config_json)

        hdx_config_yaml = kwargs.get('hdx_config_yaml', '')
        if hdx_config_found:
            if hdx_config_yaml:
                raise ConfigurationError('More than one HDX configuration given!')
        else:
            if not hdx_config_yaml:
                hdx_config_yaml = Configuration.default_hdx_config_yaml
                if isfile(hdx_config_yaml):
                    logger.info('No HDX configuration parameter. Using default configuration file: %s.' % hdx_config_yaml)
                else:
                    logger.info('No HDX configuration parameter and no configuration file at default path: %s.' % hdx_config_yaml)
                    hdx_config_yaml = None
                    hdx_config_dict = dict()
            if hdx_config_yaml:
                logger.info('Loading HDX configuration from: %s' % hdx_config_yaml)
                hdx_config_dict = load_yaml(hdx_config_yaml)

        self.data = merge_two_dictionaries(hdx_base_config_dict, hdx_config_dict)

        project_config_found = False
        project_config_dict = kwargs.get('project_config_dict', None)
        if project_config_dict is not None:
            project_config_found = True
            logger.info('Loading project configuration from dictionary')

        project_config_json = kwargs.get('project_config_json', '')
        if project_config_json:
            if project_config_found:
                raise ConfigurationError('More than one project configuration given!')
            project_config_found = True
            logger.info('Loading project configuration from: %s' % project_config_json)
            project_config_dict = load_json(project_config_json)

        project_config_yaml = kwargs.get('project_config_yaml', '')
        if project_config_found:
            if project_config_yaml:
                raise ConfigurationError('More than one project configuration given!')
        else:
            if project_config_yaml:
                logger.info('Loading project configuration from: %s' % project_config_yaml)
                project_config_dict = load_yaml(project_config_yaml)
            else:
                project_config_dict = dict()

        self.data = merge_two_dictionaries(hdx_base_config_dict, project_config_dict)

        ua = kwargs.get('full_agent')
        if ua:
            self.user_agent = ua
        else:
            try:
                self.user_agent = UserAgent.get(prefix=Configuration.prefix, **kwargs)
            except UserAgentError:
                self.user_agent = UserAgent.get(prefix=Configuration.prefix, **self.data)
        self.hdx_read_only = kwargs.get('hdx_read_only', self.data.get('hdx_read_only', False))
        logger.info('Read only access to HDX: %s' % str(self.hdx_read_only))
        self.hdx_key = kwargs.get('hdx_key', self.data.get('hdx_key'))
        if not self.hdx_key and not self.hdx_read_only:
            raise ConfigurationError('No HDX API key supplied as a parameter or in configuration!')
        hdx_url = kwargs.get('hdx_url', self.data.get('hdx_url'))
        if hdx_url:
            self.hdx_site = 'hdx_custom_site'
            hdx_url = hdx_url.rstrip('/')
            self.data[self.hdx_site] = {'url': hdx_url}
        else:
            self.hdx_site = 'hdx_%s_site' % kwargs.get('hdx_site', self.data.get('hdx_site', 'stage'))
            if self.hdx_site not in self.data:
                raise ConfigurationError('%s not defined in configuration!' % self.hdx_site)

    def set_read_only(self, read_only=True):
        # type: (bool) -> None
        """
        Set HDX read only flag

        Args:
            read_only (bool): Value to set HDX read only flag. Defaults to True.
        Returns:
            None

        """
        self.hdx_read_only = read_only

    def set_api_key(self, apikey):
        # type: (str) -> None
        """
        Set HDX api key

        Args:
            apikey (str): Value to set api key.
        Returns:
            None

        """
        self.hdx_key = apikey

    def get_api_key(self):
        # type: () -> Optional[str]
        """
        Return HDX api key or None if read only

        Returns:
            Optional[str]: HDX api key or None if read only

        """
        if self.hdx_read_only:
            return None
        return self.hdx_key

    def get_user_agent(self):
        # type: () -> str
        """
        Return user agent

        Returns:
            str: User agent

        """
        return self.user_agent

    def get_hdx_site_url(self):
        # type: () -> str
        """
        Return HDX web site url

        Returns:
            str: HDX web site url

        """
        return self.data[self.hdx_site]['url']

    def get_dataset_url(self, name):
        # type: (str) -> str
        """
        Return HDX dataset url given dataset name

        Args:
            name: Dataset name

        Returns:
            str: HDX dataset url

        """
        return '%s/dataset/%s' % (self.get_hdx_site_url(), name)

    def _get_credentials(self):
        # type: () -> Optional[Tuple[str, str]]
        """
        Return HDX site username and password

        Returns:
            Optional[Tuple[str, str]]: HDX site username and password or None

        """
        site = self.data[self.hdx_site]
        username = site.get('username')
        if username:
            return b64decode(username).decode('utf-8'), b64decode(site['password']).decode('utf-8')
        else:
            return None

    def remoteckan(self):
        # type: () -> ckanapi.RemoteCKAN
        """
        Return the remote CKAN object (see ckanapi library)

        Returns:
            ckanapi.RemoteCKAN: The remote CKAN object

        """
        if self._remoteckan is None:
            raise ConfigurationError('There is no remote CKAN set up! Use Configuration.create(**kwargs)')
        return self._remoteckan

    def call_remoteckan(self, *args, **kwargs):
        # type: (Any, Any) -> Dict
        """
        Calls the remote CKAN

        Args:
            *args: Arguments to pass to remote CKAN call_action method
            **kwargs: Keyword arguments to pass to remote CKAN call_action method

        Returns:
            Dict: The response from the remote CKAN call_action method

        """
        requests_kwargs = kwargs.get('requests_kwargs', dict())
        credentials = self._get_credentials()
        if credentials:
            requests_kwargs['auth'] = credentials
        kwargs['requests_kwargs'] = requests_kwargs
        apikey = kwargs.get('apikey', self.get_api_key())
        kwargs['apikey'] = apikey
        return self.remoteckan().call_action(*args, **kwargs)

    @staticmethod
    def _environment_variables(**kwargs):
        # type: (Any) -> Any
        """
        Overwrite keyword arguments with environment variables

        Args:
            **kwargs: See below
            hdx_url (str): HDX url to use. Overrides hdx_site.
            hdx_site (str): HDX site to use eg. prod, test. Defaults to test.
            hdx_key (str): Your HDX key. Ignored if hdx_read_only = True.

        Returns:
            kwargs: Changed keyword arguments

        """

        hdx_key = os.getenv('HDX_KEY')
        if hdx_key is not None:
            kwargs['hdx_key'] = hdx_key
        hdx_url = os.getenv('HDX_URL')
        if hdx_url is not None:
            kwargs['hdx_url'] = hdx_url
        else:
            hdx_site = os.getenv('HDX_SITE')
            if hdx_site is not None:
                kwargs['hdx_site'] = hdx_site
        return kwargs

    @classmethod
    def create_remoteckan(cls, site_url, user_agent=None, user_agent_config_yaml=None, user_agent_lookup=None,
                          use_env=False, session=None, **kwargs):
        # type: (str, Optional[str], Optional[str], Optional[str], bool, requests.Session, Any) -> ckanapi.RemoteCKAN
        """
        Create remote CKAN instance from configuration

        Args:
            site_url (str): Site url.
            user_agent (Optional[str]): User agent string. HDXPythonLibrary/X.X.X- is prefixed.
            user_agent_config_yaml (Optional[str]): Path to YAML user agent configuration. Ignored if user_agent supplied. Defaults to ~/.useragent.yml.
            user_agent_lookup (Optional[str]): Lookup key for YAML. Ignored if user_agent supplied.
            use_env (bool): Whether to read environment variables. Defaults to False.
            session (requests.Session): requests Session object to use. Defaults to calling hdx.utilities.session.get_session()

        Returns:
            ckanapi.RemoteCKAN: Remote CKAN instance

        """
        if not session:
            whitelist = frozenset(['HEAD', 'TRACE', 'GET', 'POST', 'PUT', 'OPTIONS', 'DELETE'])
            session = get_session(user_agent, user_agent_config_yaml, user_agent_lookup, use_env,
                                  prefix=Configuration.prefix, method_whitelist=whitelist, **kwargs)
            ua = session.headers['User-Agent']
        else:
            ua = kwargs.get('full_agent')
            if not ua:
                ua = UserAgent.get(user_agent, user_agent_config_yaml, user_agent_lookup, prefix=Configuration.prefix,
                                   **kwargs)
        return ckanapi.RemoteCKAN(site_url, user_agent=ua, session=session)

    def setup_remoteckan(self, remoteckan=None, **kwargs):
        # type: (Optional[ckanapi.RemoteCKAN], Any) -> None
        """
        Set up remote CKAN from provided CKAN or by creating from configuration

        Args:
            remoteckan (Optional[ckanapi.RemoteCKAN]): CKAN instance. Defaults to setting one up from configuration.

        Returns:
            None

        """
        if remoteckan is None:
            self._remoteckan = self.create_remoteckan(self.get_hdx_site_url(), full_agent=self.get_user_agent(),
                                                      **kwargs)
        else:
            self._remoteckan = remoteckan

    def emailer(self):
        # type: () -> Email
        """
        Return the Email object (see :any:`Email`)

        Returns:
            Email: The email object

        """
        if self._emailer is None:
            raise ConfigurationError('There is no emailer set up! Use setup_emailer(Any)')
        return self._emailer

    def setup_emailer(self, **kwargs):
        # type: (Any) -> None
        """
        Set up emailer. Parameters in dictionary or file (eg. yaml below):
        | connection_type: "ssl"   ("ssl" for smtp ssl or "lmtp", otherwise basic smtp is assumed)
        | host: "localhost"
        | port: 123
        | local_hostname: "mycomputer.fqdn.com"
        | timeout: 3
        | username: "user"
        | password: "pass"

        Args:
            **kwargs: See below
            email_config_dict (dict): HDX configuration dictionary OR
            email_config_json (str): Path to JSON HDX configuration OR
            email_config_yaml (str): Path to YAML HDX configuration. Defaults to ~/hdx_email_configuration.yml.

        Returns:
            None
        """
        self._emailer = Email(**kwargs)

    @classmethod
    def read(cls):
        # type: () -> 'Configuration'
        """
        Read the HDX configuration

        Returns:
            Configuration: The HDX configuration

        """
        if cls._configuration is None:
            raise ConfigurationError('There is no HDX configuration! Use Configuration.create(**kwargs)')
        return cls._configuration

    @classmethod
    def setup(cls, configuration=None, **kwargs):
        # type: (Optional['Configuration'], Any) -> None
        """
        Set up the HDX configuration

        Args:
            configuration (Optional[Configuration]): Configuration instance. Defaults to setting one up from passed arguments.
            **kwargs: See below
            user_agent (str): User agent string. HDXPythonLibrary/X.X.X- is prefixed. Must be supplied if remoteckan is not.
            user_agent_config_yaml (str): Path to YAML user agent configuration. Ignored if user_agent supplied. Defaults to ~/.useragent.yml.
            user_agent_lookup (str): Lookup key for YAML. Ignored if user_agent supplied.
            hdx_url (str): HDX url to use. Overrides hdx_site.
            hdx_site (str): HDX site to use eg. prod, test.
            hdx_read_only (bool): Whether to access HDX in read only mode. Defaults to False.
            hdx_key (str): Your HDX key. Ignored if hdx_read_only = True.
            hdx_config_dict (dict): HDX configuration dictionary to use instead of above 3 parameters OR
            hdx_config_json (str): Path to JSON HDX configuration OR
            hdx_config_yaml (str): Path to YAML HDX configuration
            project_config_dict (dict): Project configuration dictionary OR
            project_config_json (str): Path to JSON Project configuration OR
            project_config_yaml (str): Path to YAML Project configuration
            hdx_base_config_dict (dict): HDX base configuration dictionary OR
            hdx_base_config_json (str): Path to JSON HDX base configuration OR
            hdx_base_config_yaml (str): Path to YAML HDX base configuration. Defaults to library's internal hdx_base_configuration.yml.

        Returns:
            None

        """
        if configuration is None:
            cls._configuration = Configuration(**kwargs)
        else:
            cls._configuration = configuration

    @classmethod
    def _create(cls, configuration=None, remoteckan=None, **kwargs):
        # type: (Optional['Configuration'], Optional[ckanapi.RemoteCKAN], Any) -> str
        """
        Create HDX configuration

        Args:
            configuration (Optional[Configuration]): Configuration instance. Defaults to setting one up from passed arguments.
            remoteckan (Optional[ckanapi.RemoteCKAN]): CKAN instance. Defaults to setting one up from configuration.
            **kwargs: See below
            user_agent (str): User agent string. HDXPythonLibrary/X.X.X- is prefixed. Must be supplied if remoteckan is not.
            user_agent_config_yaml (str): Path to YAML user agent configuration. Ignored if user_agent supplied. Defaults to ~/.useragent.yml.
            user_agent_lookup (str): Lookup key for YAML. Ignored if user_agent supplied.
            hdx_url (str): HDX url to use. Overrides hdx_site.
            hdx_site (str): HDX site to use eg. prod, test.
            hdx_read_only (bool): Whether to access HDX in read only mode. Defaults to False.
            hdx_key (str): Your HDX key. Ignored if hdx_read_only = True.
            hdx_config_dict (dict): HDX configuration dictionary to use instead of above 3 parameters OR
            hdx_config_json (str): Path to JSON HDX configuration OR
            hdx_config_yaml (str): Path to YAML HDX configuration
            project_config_dict (dict): Project configuration dictionary OR
            project_config_json (str): Path to JSON Project configuration OR
            project_config_yaml (str): Path to YAML Project configuration
            hdx_base_config_dict (dict): HDX base configuration dictionary OR
            hdx_base_config_json (str): Path to JSON HDX base configuration OR
            hdx_base_config_yaml (str): Path to YAML HDX base configuration. Defaults to library's internal hdx_base_configuration.yml.

        Returns:
            str: HDX site url

        """
        kwargs = cls._environment_variables(**kwargs)
        cls.setup(configuration, **kwargs)
        cls._configuration.setup_remoteckan(remoteckan, **kwargs)
        return cls._configuration.get_hdx_site_url()

    @classmethod
    def create(cls, configuration=None, remoteckan=None, **kwargs):
        # type: (Optional['Configuration'], Optional[ckanapi.RemoteCKAN], Any) -> str
        """
        Create HDX configuration. Can only be called once (will raise an error if called more than once).

        Args:
            configuration (Optional[Configuration]): Configuration instance. Defaults to setting one up from passed arguments.
            remoteckan (Optional[ckanapi.RemoteCKAN]): CKAN instance. Defaults to setting one up from configuration.
            **kwargs: See below
            user_agent (str): User agent string. HDXPythonLibrary/X.X.X- is prefixed. Must be supplied if remoteckan is not.
            user_agent_config_yaml (str): Path to YAML user agent configuration. Ignored if user_agent supplied. Defaults to ~/.useragent.yml.
            user_agent_lookup (str): Lookup key for YAML. Ignored if user_agent supplied.
            hdx_url (str): HDX url to use. Overrides hdx_site.
            hdx_site (str): HDX site to use eg. prod, test.
            hdx_read_only (bool): Whether to access HDX in read only mode. Defaults to False.
            hdx_key (str): Your HDX key. Ignored if hdx_read_only = True.
            hdx_config_dict (dict): HDX configuration dictionary to use instead of above 3 parameters OR
            hdx_config_json (str): Path to JSON HDX configuration OR
            hdx_config_yaml (str): Path to YAML HDX configuration
            project_config_dict (dict): Project configuration dictionary OR
            project_config_json (str): Path to JSON Project configuration OR
            project_config_yaml (str): Path to YAML Project configuration
            hdx_base_config_dict (dict): HDX base configuration dictionary OR
            hdx_base_config_json (str): Path to JSON HDX base configuration OR
            hdx_base_config_yaml (str): Path to YAML HDX base configuration. Defaults to library's internal hdx_base_configuration.yml.

        Returns:
            str: HDX site url

        """
        if cls._configuration is not None:
            raise ConfigurationError('Configuration already created!')
        return cls._create(configuration=configuration, remoteckan=remoteckan, **kwargs)

    @classmethod
    def delete(cls):
        # type: () -> None
        """
        Delete global HDX configuration.

        Returns:
            None

        """
        cls._configuration = None



# -*'coding: UTF-8 -*-
"""HDX Location Tests"""
from hdx.location.country import Country

from hdx.hdx_configuration import Configuration
from hdx.hdx_locations import Locations


class MyConfiguration:
    def call_remoteckan(self, a, b):
        return [{'name': 'zaf', 'title': 'South Africa'}]


class TestHDXLocations:
    def test_validlocations(self, project_config_yaml):
        Country.countriesdata(use_live=False)
        validlocations = [{'name': 'shn', 'title': 'St. Helena'}]
        assert Locations.get_HDX_code_from_location('sh', locations=validlocations) is None
        assert Locations.get_HDX_code_from_location_partial('sh', locations=validlocations) == (None, False)
        assert Locations.get_location_from_HDX_code('shn', locations=validlocations) == 'St. Helena'
        validlocations = [{'name': 'zmb', 'title': 'Zambia'}, {'name': 'pry', 'title': 'Paraguay'}]
        Locations.set_validlocations(validlocations)
        assert Locations.validlocations() == validlocations
        assert Locations.get_HDX_code_from_location_partial('NOT') == (None, False)
        assert Locations.get_location_from_HDX_code('pr') is None
        assert Locations.get_HDX_code_from_location('zmb') == 'ZMB'
        assert Locations.get_HDX_code_from_location_partial('zmb') == ('ZMB', True)
        assert Locations.get_HDX_code_from_location('Z') is None
        assert Locations.get_HDX_code_from_location_partial('Z') == ('ZMB', False)
        assert Locations.get_HDX_code_from_location_partial('Zambia') == ('ZMB', True)
        assert Locations.get_HDX_code_from_location_partial('ZAM') == ('ZMB', False)
        assert Locations.get_location_from_HDX_code('zmb', locations=validlocations) == 'Zambia'
        validlocations = [{'name': 'shn', 'title': 'St. Helena'}]
        assert Locations.get_HDX_code_from_location('sh', locations=validlocations) is None
        assert Locations.get_HDX_code_from_location_partial('sh', locations=validlocations) == (None, False)
        assert Locations.get_location_from_HDX_code('shn', locations=validlocations) == 'St. Helena'
        Configuration.setup(MyConfiguration())
        Locations.set_validlocations(None)
        assert Locations.get_HDX_code_from_location('zaf') == 'ZAF'
        assert Locations.get_HDX_code_from_location_partial('zaf') == ('ZAF', True)
        assert Locations.get_location_from_HDX_code('zaf') == 'South Africa'

